@@ -12,6 +12,8 @@ __docformat__ = 'restructuredtext'
 
 import re, textwrap
 
+types = __import__('types')
+
 # for table2string
 import numpy as np
 from math import ceil
@@ -403,7 +405,7 @@ def table2string(table, out=None):
         table[i] += [''] * (Nelements_max - len(table_))
 
     # figure out lengths within each column
-    atable = np.asarray(table)
+    atable = np.asarray(table).astype(str)
     # eat whole entry while computing width for @w (for wide)
     markup_strip = re.compile('^@([lrc]|w.*)')
     col_width = [ max( [len(markup_strip.sub('', x))
@@ -443,6 +445,18 @@ def table2string(table, out=None):
         out.close()
         return value
 
+def _saferepr(f):
+    """repr which would not repr instances of bound methods since that might recurse
+
+    See: bound methods to its instances
+    https://github.com/PyMVPA/PyMVPA/issues/122
+    """
+    if type(f) == types.MethodType:
+        objid = _strid(f.im_self) if __debug__ and 'ID_IN_REPR' in debug.active else ""
+        return "<bound %s%s.%s>" % (f.im_class.__name__, objid, f.__func__.__name__)
+    else:
+        return repr(f)
+
 def _repr_attrs(obj, attrs, default=None, error_value='ERROR'):
     """Helper to obtain a list of formatted attributes different from
     the default
@@ -451,10 +465,9 @@ def _repr_attrs(obj, attrs, default=None, error_value='ERROR'):
     for a in attrs:
         v = getattr(obj, a, error_value)
         if not (v is default or isinstance(v, basestring) and v == default):
-            out.append('%s=%r' % (a, v))
+            out.append('%s=%s' % (a, _saferepr(v)))
     return out
 
-
 def _repr(obj, *args, **kwargs):
     """Helper to get a structured __repr__ for all objects.
 
@@ -489,6 +502,10 @@ def _repr(obj, *args, **kwargs):
 
     return "%s(%s)" % (cls_name, auto_repr)
 
+def _strid(obj):
+    """Helper for centralized string id representation in debug msgs
+    """
+    return "#%d" % (id(obj))
 
 def _str(obj, *args, **kwargs):
     """Helper to get a structured __str__ for all objects.
@@ -536,7 +553,7 @@ def _str(obj, *args, **kwargs):
             if s[-1]:
                 s += ','
             s += ' '
-        s += 'id=%i' % id(obj)
+        s += _strid(obj)
 
     # finally wrap in <> and return
     # + instead of '%s' for bits of speedup
