@@ -321,7 +321,7 @@ def version_to_tuple(v):
     of numerics and alpha numbers
     """
     if isinstance(v, basestring):
-        v = v.split('.')
+        v = map(str, v.split('.'))
     elif isinstance(v, tuple) or isinstance(v, list):
         # assure tuple
         pass
@@ -367,7 +367,15 @@ class SmartVersion(Version):
     So here is an ad-hoc and not as nice implementation
     """
 
+    def __reduce__(self):
+        """Rudimentary __reduce__ because Version is not derived from object"""
+        # parent class Version might not even assign any vstring when empty
+        return self.__class__, (getattr(self, 'vstring', ''),)
+
     def parse(self, vstring):
+        # Unicode gives grief on older releases and anyway arguably comparable
+        if isinstance(vstring, unicode):
+            vstring = str(vstring)
         self.vstring = vstring
         self.version = version_to_tuple(vstring)
 
@@ -381,7 +389,7 @@ class SmartVersion(Version):
             return ""
 
     def __cmp__(self, other):
-        if isinstance(other, (str, tuple, list)):
+        if isinstance(other, (str, unicode, tuple, list)):
             other = SmartVersion(other)
         elif isinstance(other, SmartVersion):
             pass
@@ -719,7 +727,7 @@ def get_limit_filter(limit, collection):
     Parameters
     -----------
     limit : None or str or dict
-      If ``None`` all elements wil be included in the filter. If an single
+      If ``None`` all elements will be included in the filter. If an single
       attribute name is given, its unique values will be used to define
       chunks of data that are marked in the filter as unique integers. Finally,
       if a dictionary is provided, its keys define attribute names and its
@@ -735,7 +743,7 @@ def get_limit_filter(limit, collection):
     -------
     array
       This array is either boolean, where a `True` elements represent including
-      in the filter, or the array is numerical, where it unqiue integer values
+      in the filter, or the array is numerical, where it unique integer values
       defines individual chunks of a filter.
     """
     attr_length = collection.attr_length
