@@ -75,6 +75,9 @@ class CrossValidationTests(unittest.TestCase):
                                   space='partitions'))
         results = cv(data)
 
+        # results must not be the same
+        self.assertTrue(len(np.unique(results.samples))>1)
+
         # must be at chance level
         pmean = np.array(results).mean()
         self.assertTrue( pmean < 0.58 and pmean > 0.42 )
@@ -89,10 +92,11 @@ class CrossValidationTests(unittest.TestCase):
         assert_raises(ValueError, cv, data)
 
 
-def suite():
+def suite():  # pragma: no cover
     return unittest.makeSuite(CrossValidationTests)
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import runner
+    runner.run()
 
